﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using OSGeo.FDO.ClientServices;
using OSGeo.FDO.Commands.DataStore;
using System.Diagnostics;
using OSGeo.FDO.Commands.SpatialContext;
using OSGeo.FDO.Schema;
using OSGeo.FDO.Commands.Schema;
using System.IO;

namespace FdoSqliteTest
{
    class Program
    {
        static void Main(string[] args)
        {
            if (File.Exists("Test.sqlite"))
                File.Delete("Test.sqlite");

            var conn = FeatureAccessManager.GetConnectionManager().CreateConnection("OSGeo.SQLite");

            using (var cds = (ICreateDataStore)conn.CreateCommand(OSGeo.FDO.Commands.CommandType.CommandType_CreateDataStore))
            {
                cds.DataStoreProperties.SetProperty("File", "Test.sqlite");
                cds.Execute();
            }

            conn.ConnectionString = "File=Test.sqlite";
            conn.Open();

            Debug.Assert(conn.ConnectionState == OSGeo.FDO.Connections.ConnectionState.ConnectionState_Open);

            string SPATIAL_CONTEXT_NAME = "MySpatialContext";
            using (var csc = (ICreateSpatialContext)conn.CreateCommand(OSGeo.FDO.Commands.CommandType.CommandType_CreateSpatialContext))
            {
                //All Spatial Context Associations will always default to the first spatial context regardless
                csc.Name = "Default";
                csc.CoordinateSystem = "XY-M";
                csc.CoordinateSystemWkt = "LOCAL_CS[\"Non-Earth (Meter)\", LOCAL_DATUM[\"Local Datum\", 0], UNIT[\"Meter\", 1], AXIS[\"X\", EAST], AXIS[\"Y\", NORTH]]";
                csc.Description = "Default Spatial Context";
                csc.ExtentType = SpatialContextExtentType.SpatialContextExtentType_Dynamic;
                csc.Execute();

                csc.Name = SPATIAL_CONTEXT_NAME;
                csc.CoordinateSystem = "LL84";
                csc.CoordinateSystemWkt = "GEOGCS[\"LL84\",DATUM[\"WGS84\",SPHEROID[\"WGS84\",6378137.000,298.25722293]],PRIMEM[\"Greenwich\",0],UNIT[\"Degree\",0.01745329251994]]";
                csc.Description = "My Spatial Context";
                csc.ExtentType = SpatialContextExtentType.SpatialContextExtentType_Dynamic;
                csc.Execute();
            }

            var scNames = new List<string>();
            using (var gsc = (IGetSpatialContexts)conn.CreateCommand(OSGeo.FDO.Commands.CommandType.CommandType_GetSpatialContexts))
            {
                using (var reader = gsc.Execute())
                {
                    while (reader.ReadNext())
                    {
                        scNames.Add(reader.GetName());
                    }
                }
            }

            Debug.Assert(scNames.Count == 2);
            Debug.Assert(scNames.Contains(SPATIAL_CONTEXT_NAME));

            var schema = new FeatureSchema("MySchema", "");
            var cls = new FeatureClass("TestClass", "");
            var id = new DataPropertyDefinition("ID", "") 
            {
                DataType = DataType.DataType_Int32,
                IsAutoGenerated = true,
            };
            cls.Properties.Add(id);
            cls.IdentityProperties.Add(id);

            var geom = new GeometricPropertyDefinition("Geometry", "");
            geom.GeometryTypes = (int)GeometricType.GeometricType_All;
            geom.SpatialContextAssociation = SPATIAL_CONTEXT_NAME;

            cls.Properties.Add(geom);
            cls.GeometryProperty = geom;

            schema.Classes.Add(cls);

            using(var apply = (IApplySchema)conn.CreateCommand(OSGeo.FDO.Commands.CommandType.CommandType_ApplySchema))
            {
                apply.FeatureSchema = schema;
                apply.Execute();
            }

            using (var desc = (IDescribeSchema)conn.CreateCommand(OSGeo.FDO.Commands.CommandType.CommandType_DescribeSchema))
            {
                using(var schemas = desc.Execute())
                {
                    var fc = schemas[0].Classes[0] as FeatureClass;
                    Debug.Assert(fc != null);

                    //This will always be the first *created* spatial context (Default). Assertion fails.
                    //Assertion passes if the order in which the spatial contexts were created was reversed. (SPATIAL_CONTEXT_NAME, then Default)
                    Debug.Assert(fc.GeometryProperty.SpatialContextAssociation == SPATIAL_CONTEXT_NAME);
                }
            }

            conn.Close();
        }
    }
}
